@@ -48,14 +48,23 @@ public:
     powerSavingUnknown
   };
 
+  enum Encryption {
+    cryptNone    = 1 << 0,
+    cryptWEP     = 1 << 1,
+    cryptWPA_PSK = 1 << 2,
+    cryptWPA_EAP = 1 << 3,
+    cryptWPA2    = 1 << 4
+  };
+
   class Network
   {
   public:
-    std::wstring essid;
-    Type         type;
-    unsigned     bitrate;
-    int          channel;
-    int          quality;
+    std::wstring  essid;
+    Type          type;
+    unsigned      bitrate;
+    unsigned long encryption;
+    int           channel;
+    int           quality;
 
     Network()
     {
@@ -66,6 +75,7 @@ public:
     {
       type=typeNone;
       bitrate=0;
+      encryption=0;
       quality=-1;
       channel=-1;
     }
