@@ -29,6 +29,7 @@
 #include <Lum/Def/MultiView.h>
 
 #include <Lum/Dlg/About.h>
+#include <Lum/Dlg/Msg.h>
 
 #include <Lum/Model/Action.h>
 #include <Lum/Model/Boolean.h>
@@ -59,6 +60,7 @@
   #include "StatusNone.h"
 #endif
 
+#include "Configuration.h"
 #include "ConfigDialog.h"
 
 #define NETWORK_REFRESH    10
@@ -70,47 +72,52 @@ static Lum::Def::AppInfo info;
 typedef Lum::Model::StdTable<Status::Network> NetworkModel;
 typedef Lum::Base::Reference<NetworkModel>    NetworkModelRef;
 
-class SignalLevel : public Lum::Control
+class NetworkObject : public Lum::Object
 {
 private:
-  Lum::Model::DoubleRef model;
-  Lum::OS::FontRef      font;
-  Lum::OS::ColorRef     red;
-  Lum::OS::ColorRef     green;
+  Lum::OS::FontRef  font;
+  Lum::OS::ColorRef red;
+  Lum::OS::ColorRef green;
+  Lum::OS::ColorRef yellow;
+  Lum::OS::ColorRef grey;
+  Status::Network   network;
 
 public:
-  SignalLevel()
+  NetworkObject()
   : font(Lum::OS::display->GetFont()),
     red(1,0,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
-    green(0,1,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor))
+    green(0,1,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
+    yellow(1,0.5,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
+    grey(0.7,0.7,0.7,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor))
   {
     // no code
   }
 
-  bool SetModel(Lum::Base::Model* model)
+  void SetNetwork(const Status::Network& network)
   {
-    this->model=dynamic_cast<Lum::Model::Double*>(model);
-
-    Control::SetModel(this->model);
-
-    return this->model.Valid();
+    this->network=network;
   }
 
   void CalcSize()
   {
-    minWidth=2*Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder)+
-             3*font->StringWidth(L"m",Lum::OS::Font::normal);
-    minHeight=font->pixelHeight+2*Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
+    minWidth=font->height+
+             Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder)+
+             3*font->StringWidth(L"m");
+    minHeight=Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+              font->height+
+              Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+              font->height+
+              Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
 
     width=minWidth;
     height=minHeight;
 
-    Control::CalcSize();
+    Object::CalcSize();
   }
 
   void Draw(int x, int y, size_t w, size_t h)
   {
-    Control::Draw(x,y,w,h);
+    Object::Draw(x,y,w,h);
 
     if (!OIntersect(x,y,w,h)) {
       return;
@@ -120,11 +127,69 @@ public:
 
     Lum::OS::DrawInfo *draw=GetDrawInfo();
 
-    if (!model.Valid() || model->IsNull()) {
-      return;
+    //
+    // Network security type
+    //
+
+    if (network.encryption!=0) {
+      if (network.encryption & Status::cryptNone) {
+        draw->PushForeground(green);
+      }
+      else if (network.encryption & Status::cryptWEP) {
+        draw->PushForeground(yellow);
+      }
+      else if (network.encryption & Status::cryptWPA_PSK) {
+        draw->PushForeground(red);
+      }
+      else if (network.encryption & Status::cryptWPA_EAP) {
+        draw->PushForeground(red);
+      }
+      else if (network.encryption & Status::cryptWPA2) {
+        draw->PushForeground(red);
+      }
+    }
+    else {
+      draw->PushForeground(grey);
     }
 
-    double percent=model->Get();
+    draw->FillRectangle(this->x,
+                        this->y+Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder),
+                        font->height,font->height);
+    draw->PopForeground();
+
+    //
+    // Name of network
+    //
+
+    draw->PushFont(font);
+    draw->PushForeground(parent->GetTextColor(draw));
+    draw->DrawString(this->x+font->height+Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder),
+                     this->y+font->ascent,
+                     network.essid);
+    draw->PopForeground();
+    draw->PopFont();
+
+    //
+    // Channel number
+    //
+
+    if (network.channel>=0) {
+      std::wstring channel=L"Channel: "+Lum::Base::NumberToWString(network.channel);
+
+      draw->PushFont(font);
+      draw->PushForeground(parent->GetTextColor(draw));
+      draw->DrawString(this->x+width-font->StringWidth(channel)+1,
+                       this->y+font->ascent,
+                       channel);
+      draw->PopForeground();
+      draw->PopFont();
+    }
+
+    //
+    // Signal strength
+    //
+
+    double percent=network.quality/100.0;
 
     if (percent<=0.2) {
       draw->PushForeground(red);
@@ -133,7 +198,11 @@ public:
       draw->PushForeground(green);
     }
 
-    draw->FillRectangle(this->x,this->y+1,
+    draw->FillRectangle(this->x,
+                        this->y+
+                        Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+                        font->height+
+                        Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder),
                         (size_t)(width*percent),height-2);
 
 
@@ -148,156 +217,44 @@ public:
     draw->PushForeground(Lum::OS::Display::textColor);
     draw->PushFont(font,0);
     draw->DrawString(this->x+(width-extent.width+extent.left+extent.right)/2,
-                     this->y+(height-font->height)/2+font->ascent,
+                     this->y+
+                     Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+                     font->height+
+                     Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+                     font->ascent,
                      label);
     draw->PopFont();
     draw->PopForeground();
   }
 
-  void Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
+  static NetworkObject* Create(bool horizFlex, bool vertFlex)
   {
-    if (model==this->model) {
-      Redraw();
-    }
-    Control::Resync(model,msg);
-  }
+    NetworkObject *object=new NetworkObject();
 
-  static SignalLevel* Create(bool horizFlex, bool vertFlex)
-  {
-    SignalLevel *control=new SignalLevel();
+    object->SetFlex(horizFlex,vertFlex);
 
-    control->SetFlex(horizFlex,vertFlex);
-
-    return control;
-  }
-};
-
-class Encryption : public Lum::Control
-{
-private:
-  Lum::Model::SizeTRef model;
-  Lum::OS::FontRef     font;
-  Lum::OS::ColorRef    grey;
-  Lum::OS::ColorRef    green;
-  Lum::OS::ColorRef    yellow;
-  Lum::OS::ColorRef    red;
-
-public:
-  Encryption()
-  : font(Lum::OS::display->GetFont()),
-    grey(0.7,0.7,0.7,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
-    green(0,1,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
-    yellow(1,0.5,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
-    red(1,0,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor))
-  {
-    // no code
-  }
-
-  bool SetModel(Lum::Base::Model* model)
-  {
-    this->model=dynamic_cast<Lum::Model::SizeT*>(model);
-
-    Control::SetModel(this->model);
-
-    return this->model.Valid();
-  }
-
-  void CalcSize()
-  {
-    minHeight=font->pixelHeight+2*Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
-    minWidth=minHeight;
-
-    width=minWidth;
-    height=minHeight;
-
-    Control::CalcSize();
-  }
-
-  void Draw(int x, int y, size_t w, size_t h)
-  {
-    Control::Draw(x,y,w,h);
-
-    if (!OIntersect(x,y,w,h)) {
-      return;
-    }
-
-    /* --- */
-
-    Lum::OS::DrawInfo *draw=GetDrawInfo();
-
-    if (!model.Valid() || model->IsNull()) {
-      draw->PushForeground(grey);
-      draw->FillRectangle(this->x+(width-height)/2,this->y+1,
-                          height,height-2);
-      draw->PopForeground();
-      return;
-    }
-
-    size_t state=model->Get();
-
-    switch (state) {
-    case 0:
-      draw->PushForeground(green);
-      break;
-    case 1:
-      draw->PushForeground(yellow);
-      break;
-    case 2:
-      draw->PushForeground(red);
-      break;
-    default:
-      assert(false);
-    }
-
-    draw->FillRectangle(this->x+(width-height)/2,this->y+1,
-                        height,height-2);
-    draw->PopForeground();
-  }
-
-  void Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
-  {
-    if (model==this->model) {
-      Redraw();
-    }
-    Control::Resync(model,msg);
-  }
-
-  static Encryption* Create(bool horizFlex, bool vertFlex)
-  {
-    Encryption *control=new Encryption();
-
-    control->SetFlex(horizFlex,vertFlex);
-
-    return control;
+    return object;
   }
 };
 
 class NetworkDataProvider : public NetworkModel::DataProvider
 {
 private:
-  SignalLevel           *gauge;
-  Encryption            *encrypt;
-  Lum::Model::DoubleRef signalLevel;
-  Lum::Model::SizeTRef  encryption;
+  NetworkObject *object;
 
 public:
   NetworkDataProvider()
-  : gauge(SignalLevel::Create(true,false)),
-    encrypt(Encryption::Create(true,false)),
-    signalLevel(new Lum::Model::Double()),
-    encryption(new Lum::Model::SizeT())
+  : object(NetworkObject::Create(true,false))
   {
-    signalLevel->SetRange(0,1);
-    gauge->SetModel(signalLevel);
-
-    encrypt->SetModel(encryption);
+    // no code
   }
 
   virtual ~NetworkDataProvider()
   {
-    delete gauge;
+    delete object;
   }
 
+  /*
   std::wstring GetString(const NetworkModel::Iterator& iter, size_t column) const
   {
     switch (column) {
@@ -313,39 +270,12 @@ public:
     default:
       return L"";
     }
-  }
+  } */
 
   Lum::Object* GetObject(const NetworkModel::Iterator& iter, size_t column) const
   {
-    switch (column) {
-    case 3:
-      if (iter->encryption!=0) {
-        if (iter->encryption & Status::cryptNone) {
-          encryption->Set(0);
-        }
-        else if (iter->encryption & Status::cryptWEP) {
-          encryption->Set(1);
-        }
-        else if (iter->encryption & Status::cryptWPA_PSK) {
-          encryption->Set(2);
-        }
-        else if (iter->encryption & Status::cryptWPA_EAP) {
-          encryption->Set(2);
-        }
-        else if (iter->encryption & Status::cryptWPA2) {
-          encryption->Set(2);
-        }
-      }
-      else {
-        encryption->SetNull();
-      }
-      return encrypt;
-    case 4:
-      signalLevel->Set(iter->quality/100.0);
-      return gauge;
-    default:
-      return NULL;
-    }
+    object->SetNetwork(*iter);
+    return object;
   }
 };
 
@@ -651,7 +581,7 @@ public:
     powerSaving(new Lum::Model::Boolean()),
     refreshTime(new Lum::Model::Int(5)),
     tableModel(new NetworkModel(new NetworkDataProvider())),
-    scanForNetworks(new Lum::Model::Boolean(false)),
+    scanForNetworks(new Lum::Model::Boolean()),
     wardrive(new Lum::Model::Boolean(false)),
     powerSavingOnAction(new Lum::Model::Action()),
     powerSavingOffAction(new Lum::Model::Action()),
@@ -681,12 +611,15 @@ public:
     Observe(networksTimer);
     Observe(networksErrorTimer);
     Observe(GetOpenedAction());
+    Observe(GetClosedAction());
     Observe(configAction);
     Observe(aboutAction);
 
     Observe(status.statusChangedAction);
     Observe(status.networksChangedAction);
 
+    scanForNetworks->Set(false);
+
     Lum::Base::Path openNetworkSoundPath;
 
     openNetworkSoundPath.SetNativeDir(Lum::Base::Path::GetCWD());
@@ -731,6 +664,34 @@ public:
         scanNetworkSound->SetFilename(scanNetworkSoundPath.GetPath(),Lum::Audio::PlaySound::typeWav);
       }
     }
+
+    // For testing purposes
+
+    /*
+    status.networks.clear();
+
+    Status::Network network;
+
+    network.essid=L"Test 1";
+    network.quality=90;
+    network.channel=6;
+    network.type=Status::typeInfrastructure;
+    network.encryption=0;
+    status.networks.push_back(network);
+
+    network.essid=L"Test 2";
+    network.quality=100;
+    network.channel=6;
+    network.type=Status::typeInfrastructure;
+    network.encryption=Status::cryptWPA2;
+    status.networks.push_back(network);
+
+    network.essid=L"Test 3";
+    network.quality=10;
+    network.channel=6;
+    network.type=Status::typeInfrastructure;
+    network.encryption=Status::cryptWPA2;
+    status.networks.push_back(network);*/
   }
 
   ~MyWindow()
@@ -795,10 +756,11 @@ public:
       selection=new Lum::Model::SingleLineSelection;
 
       headerModel=new Lum::Model::HeaderImpl();
-      headerModel->AddColumn(L"ESSID",Lum::Base::Size::stdCharWidth,15,true);
+      headerModel->AddColumn(L"Network",Lum::Base::Size::stdCharWidth,15,true);
+      /**
       headerModel->AddColumn(L"C",Lum::Base::Size::stdCharWidth,2);
       headerModel->AddColumn(L"Encrypt",Lum::Base::Size::stdCharWidth,7);
-      headerModel->AddColumn(L"Quality",Lum::Base::Size::stdCharWidth,10);
+      headerModel->AddColumn(L"Quality",Lum::Base::Size::stdCharWidth,10);*/
 
       table=new Lum::Table();
       table->SetFlex(true,true);
@@ -807,7 +769,7 @@ public:
       //table->SetSelection(selection);
       //table->SetSelectionAction(showDetail);
       //table->SetDoubleClickAction(showDetail);
-      table->SetShowHeader(true);
+      table->SetShowHeader(false);
       table->GetTableView()->SetMinHeight(Lum::Base::Size::stdCharHeight,4);
       table->GetTableView()->SetAutoFitColumns(true);
       table->GetTableView()->SetAutoHSize(true);
@@ -846,26 +808,48 @@ public:
 
   void Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
   {
-    if (model==GetOpenedAction() && GetOpenedAction()->IsFinished()) {
+    if (model==GetOpenedAction() &&
+        GetOpenedAction()->IsFinished()) {
       ParseArguments();
       UpdateStatus();
       RequestStatus();
+
+      if (!LoadConfig()) {
+        Lum::Dlg::Msg::ShowOk(this,
+                              L"Power management warning!",
+                              L"This application periodically polls the WLAN stack of\n"
+                              L"the OS while running (even if in background!).\n"
+                              L"\n"
+                              L"If this applications runs on an device in battery mode\n"
+                              L"this will result in more than usual battery draining,\n"
+                              L"potentially significantly reducing the time the device\n"
+                              L"can run on battery!");
+      }
+    }
+    if (model==GetClosedAction() &&
+        GetClosedAction()->IsFinished()) {
+      SaveConfig();
     }
     else if (model==scanForNetworks) {
-      if (*scanForNetworks) {
+      if (!scanForNetworks->IsNull() &&
+          *scanForNetworks) {
+        tableModel->SetEmptyText(L"(No networks found)");
         RequestNetworks();
         wardrive->Enable();
       }
       else {
+        tableModel->SetEmptyText(L"(Scanning not enabled)");
         CancelNetworks();
         UpdateNetworks();
         wardrive->Disable();
       }
     }
-    else if (model==powerSavingOnAction && powerSavingOnAction->IsFinished()) {
+    else if (model==powerSavingOnAction &&
+             powerSavingOnAction->IsFinished()) {
       status.SetPowerSaving(true);
     }
-    else if (model==powerSavingOffAction && powerSavingOffAction->IsFinished()) {
+    else if (model==powerSavingOffAction &&
+             powerSavingOffAction->IsFinished()) {
       status.SetPowerSaving(false);
     }
     else if ((model==statusTimer && statusTimer->IsFinished()) ||
@@ -880,12 +864,14 @@ public:
       std::cout << "Refreshing: " << refreshTime->Get() << std::endl;
       RequestStatus();
     }
-    else if (model==status.statusChangedAction && status.statusChangedAction->IsFinished()) {
+    else if (model==status.statusChangedAction &&
+             status.statusChangedAction->IsFinished()) {
       Lum::OS::display->RemoveTimer(statusErrorTimer);
       UpdateStatus();
       Lum::OS::display->AddTimer(refreshTime->Get(),0,statusTimer);
     }
-    else if (model==status.networksChangedAction && status.networksChangedAction->IsFinished()) {
+    else if (model==status.networksChangedAction &&
+             status.networksChangedAction->IsFinished()) {
       Lum::OS::display->RemoveTimer(networksErrorTimer);
       UpdateNetworks();
       Lum::OS::display->AddTimer(NETWORK_REFRESH,0,networksTimer);
